grammar Letrec;

prog returns [Expr value]
    :   e1=expr { $value = $e1.value; } 
    ;

expr returns [Expr value]
    :  NUMBER
        { $value = new ExprConst(Integer.parseInt($NUMBER.text)); }

    |  IDENTIFIER
        { $value = new ExprVar($IDENTIFIER.text); }

    | '-' '(' e1=expr ',' e2=expr ')'   
        { $value = new ExprDiff($e1.value, $e2.value); }

    |  'zero?' '(' e1=expr ')'   
        { $value = new ExprZeroTest($e1.value); }

    |  'if' e1=expr 'then' e2=expr 'else' e3=expr
        { $value = new ExprIf($e1.value, $e2.value, $e3.value); }

    |  'let' IDENTIFIER '=' e1=expr 'in' e2=expr
        { $value = new ExprLet($IDENTIFIER.text, $e1.value, $e2.value); }

    |  'proc' '(' IDENTIFIER ')' e1=expr       
        { $value = new ExprProc($IDENTIFIER.text, $e1.value); }

    |  '(' e1=expr e2=expr ')'      
        { $value = new ExprCall($e1.value, $e2.value); }

    |  'letrec' v1=IDENTIFIER '(' v2=IDENTIFIER ')' '=' e1=expr 'in' e2=expr  
        { $value = new ExprLetrec($v1.text, $v2.text, $e1.value, $e2.value); }

    |  'try' e1=expr 'catch' '(' IDENTIFIER ')' e2=expr
        { $value = new ExprTry($e1.value, $IDENTIFIER.text, $e2.value); }

    |  'raise' e1=expr          
        { $value = new ExprRaise($e1.value); }
    ;

fragment LETTER     :  ('a'..'z' | 'A'..'Z') ;
fragment DIGIT      :  '0'..'9' ;

IDENTIFIER :  LETTER (LETTER | DIGIT | '_' | '-' | '?')* ;
NUMBER     : '-'? DIGIT+ ;

WS         :  (' ' | '\t' | '\r' | '\n')+     { skip(); } ;
COMMENT    :  '%' ~('\n' | '\r')* '\r'? '\n' { skip(); } ;
